#!/usr/bin/python
#------------------------------------------------------------------------------
# Copyright (c) 2007-2014 Perforce Software, Inc.  Provided for use as defined in
# the Perforce Consulting Services Agreement.
#------------------------------------------------------------------------------

"""
Usage:
  python checkusers.py

This script will generate a list of all user names that are listed in any group,
but do not have a user account on the server. The results are written to removeusersfromgroups.txt.
You can pass that to removeuserfromgroups.py to do the cleanup.
"""

import os
import re
import string
import sys
import platform
import ConfigParser

if len(sys.argv) > 1:
  P4INSTANCE = str(sys.argv[1])
else:
  P4INSTANCE = '1'

config = ConfigParser.RawConfigParser()
config.read('maintenance.cfg')

##########################################################################
#####                                                                #####
#####  CONFIGURATION VARIABLES: Modify in maintenance.cfg as needed. #####
#####                                                                #####

server = (config.get(P4INSTANCE, 'server'))
p4user = (config.get(P4INSTANCE, 'p4user'))
passwd = (config.get(P4INSTANCE, 'passwd'))

if platform.system() == "Windows": 
  p4="p4.exe -p %s -u %s" % (server, p4user)
else:
  p4="/p4/1/bin/p4_1  -p %s -u %s" % (server, p4user)

os.system('echo %s| %s login' % (passwd, p4))

users = []
groupusers = []
groups = []

def main():
  try:
    for user in os.popen( "%s users -a" % p4 ).readlines():
      user = re.sub( r" <.*", r"", user )
      user = user.strip()
      user = user.lower()
      users.append( user )
  except:
    print("Non unicode user name on server. Some user names may have been skipped.")

  for group in os.popen("%s groups" % p4).readlines():
    group = group.strip()
    groups.append(group)

  for group in groups:
    pastOwners = 0
    for user in os.popen("%s group -o %s" % (p4, group)).readlines():
      user = user.strip()
      if re.match(r"Owners:", user):
        pastOwners = 1
        continue
      if pastOwners == 1:
        user = user.lower()
        if user in groupusers:
          continue
        else:
          if not re.search("users:", user):
            if user:
              groupusers.append(user)

  groupusers.sort()

  userfile = open("removeusersfromgroups.txt", "w")

  for groupuser in groupusers:
    if groupuser in users:
      continue
    else:
      userfile.write("%s\n" % groupuser)

  userfile.close()

###############################################################################
# main
if __name__ == "__main__":
  main()
